% Import 2D data from Abaqus 2D FE model and convert to Matlab format
%
% Date - Version - Additions
% 26/03/2020 - Ver1 - First code
%
%
%-------------------------------------------------------------------------%
% Notes:
% N/A
%-------------------------------------------------------------------------%
% Steps:
%
% 1) Run Abaqus FE simulation, output data. 
%
% 2) Run main_initFESimProcessingParams_'jobFileName'_vX.m
%
% 3) Read in vectorised displacement data
%
% 4) Interpolate to the nodes with scatteredInterpolant
%
% 5) Interpolate to the centroids with scatteredInterpolant
%
% 6) Save data for later use in FE data processing codes
%
%-------------------------------------------------------------------------%

clear all
close all
clc

%-------------------------------------------------------------------------%
% Load the FE simulation processing parameter file
%-------------------------------------------------------------------------%
hardCodePath = 1;

if ~hardCodePath
    [initFile,initPath] = uigetfile({'*.*','All Files'},'Select the FE initialisation file');
else
    initPath = 'E:\experimentalData\ModulusPaper\FE_UD45\AbaqusToMatlab\';
    initFile = 'FESimProcessingParameters_v1.mat';
end

% Load the file
fprintf('Loading FE processing parameters file.\n')
load([initPath,initFile])

% Enter simulation designation
jobFileName = '2D_CFRPPulse_UD45';

%% -----------------------------------------------------------------------%
% Read in data
%-------------------------------------------------------------------------%

% Read in node locs file
filename = ([extractedDataDir,'\',jobFileName,'_NodeLocs','.rpt']);
fid = fopen(filename);
mesh.nodeLocations_cell = textscan(fid, '%f%f%f', 'HeaderLines',21, 'Delimiter', ' ','MultipleDelimsAsOne',1);
mesh.nodeLocations_vec = cell2mat(mesh.nodeLocations_cell);
fclose(fid);

%% -----------------------------------------------------------------------%
% FE mesh
%-------------------------------------------------------------------------%

% Create vectors of the x and y node locations. Note that the origin is on 
% the free-edge at the centre of the sample, as programmed in the Python 
% script for the Abaqus simulation. 
mesh.xOriginalVec = 0 : FE.specElemSize : FE.specLength;
mesh.yOriginalVec = -FE.specHeight/2 : FE.specElemSize : FE.specHeight/2;

% Specify number of surface nodes for the interpolation
mesh.numberOfSurfaceNodes = length(mesh.xOriginalVec)*length(mesh.yOriginalVec);

% Read in the displacements
for m = 1:FE.specOutputFrames
    filename = ([jobFileName,'_Disp',num2str(m),'.rpt']);
    fid = fopen(filename);
    mesh.disp_cell{m} = textscan(fid, '%f%f%f', 'HeaderLines',21, 'Delimiter', 't','MultipleDelimsAsOne',1);
    mesh.disp_AllVec(:,:,m) = cell2mat(mesh.disp_cell{1,m});
    mesh.disp_xVec(:,m) = mesh.disp_AllVec(1:mesh.numberOfSurfaceNodes,2,m);
    mesh.disp_yVec(:,m) = mesh.disp_AllVec(1:mesh.numberOfSurfaceNodes,3,m);
    fclose(fid);
end

%% -----------------------------------------------------------------------%
% Interpolate to the nodes
%-------------------------------------------------------------------------%

% Rows of P contain the x,y coordinates of vi and vj
% Shift the origin to the free-edge, lower edge of specimens corner
temp.x = mesh.nodeLocations_vec(1:mesh.numberOfSurfaceNodes, 2);
temp.y = mesh.nodeLocations_vec(1:mesh.numberOfSurfaceNodes, 3) + FE.specHeight/2;

% Create vectors of the node locations to interpolate to.
mesh.x = 0 : FE.specElemSize : FE.specLength;
mesh.y = 0 : FE.specElemSize : FE.specHeight;

% Meshgrid of the nodes - 2D
[mesh.xGrid, mesh.yGrid] = meshgrid(mesh.x, mesh.y);

% Interpolate the vectorised displacements (with their associated node locations)
% to the sample NODE locations.
for m = 1:FE.specOutputFrames
    
    fprintf(['Interpolating vectorised FE displacements to the FE sample nodes for frame ',num2str(m),' \n'])
    % disp.x
    temp.vi = mesh.disp_xVec(:,m);
    temp.Fi = scatteredInterpolant(temp.x, temp.y, temp.vi);
    temp.Vqi = temp.Fi(mesh.xGrid, mesh.yGrid);
    disp.xNodal(:,:,m) = temp.Vqi;   
    
    % disp.y
    temp.vj = mesh.disp_yVec(:,m);   
    temp.Fj = scatteredInterpolant(temp.x, temp.y, temp.vj);
    temp.Vqj = temp.Fj(mesh.xGrid, mesh.yGrid);
    disp.yNodal(:,:,m) = temp.Vqj;
    
end


%% -----------------------------------------------------------------------%
% Interpolate to the centroids
%-------------------------------------------------------------------------%

% Vector of centroid locations, starting at lower left corner
mesh.xCentroid = FE.specElemSize/2 : FE.specElemSize : FE.specLength-FE.specElemSize/2;
mesh.yCentroid = FE.specElemSize/2 : FE.specElemSize : FE.specHeight-FE.specElemSize/2;
 
% Meshgrid of centroid vectors
[mesh.xGridCentroid, mesh.yGridCentroid] =...
    meshgrid(mesh.xCentroid, mesh.yCentroid);
 
% Interpolate the vectorised displacements (with their associated node locations)
% to the sample CENTROID locations.
for m = 1:FE.specOutputFrames
    
    fprintf(['Interpolating vectorised FE displacements to the FE sample centroids for frame ',num2str(m),' \n'])
    % disp.x
    temp.vi = mesh.disp_xVec(:,m);
    temp.Fi = scatteredInterpolant(temp.x,temp.y,temp.vi);
    temp.VqiCentroid = temp.Fi(mesh.xGridCentroid, mesh.yGridCentroid);
    disp.xFE(:,:,m) = temp.VqiCentroid;
    
    % disp.y
    temp.vj = mesh.disp_yVec(:,m);
    temp.Fj = scatteredInterpolant(temp.x,temp.y,temp.vj);
    temp.VqjCentroid = temp.Fj(mesh.xGridCentroid, mesh.yGridCentroid);
    disp.yFE(:,:,m) = temp.VqjCentroid;

end

% Save disp struct to dispFE (different struct names for post-processing)
dispFE = disp;

%% -----------------------------------------------------------------------%
% Save
%-------------------------------------------------------------------------%

% Save Data
fprintf('Saving matlab file containing the FE displacements. \n')
save([savePath,'\','FESimKinematicFields_',jobFileName,'.mat'],'dispFE');
