#////////////////////////////////////////////////////////////////////////////
# Explicit FE Model - 2D - Pressure Pulse on CFRP Plate
# Author: Lloyd Fletcher
# Adapted from python code developed by Alex Guigue
# Date: 22/1/2019
# Modified by Sam Parry 12/03/2020
#////////////////////////////////////////////////////////////////////////////

#----------------------------------------------------------------------------
# Preamble

from abaqus import *
from abaqusConstants import *
from caeModules import *
from driverUtils import executeOnCaeStartup
executeOnCaeStartup()
import visualization 
import displayGroupOdbToolset as dgo 
import regionToolset
import os 
import section
import step
import job
import material
import sys
import time
import math

#----------------------------------------------------------------------------
# File paths and working directory

directory='E:/experimentalData/ModulusPaper/'
extension='FE_UD45/'
jobFileName='2D_CFRPPulse_UD45'

# If the directory does not exist, create it
if not os.path.exists(directory+extension):
    os.makedirs(directory+extension) # Make the directory if need be

# Set the working directory
os.chdir(directory+extension)

#----------------------------------------------------------------------------
# Model parameters

# SIMULATION FLAGS
submitJob = 1
extractData = 1
extractDispEnerOnly = 0
outputGlobalCoords = 1

# SIMULATION PARAMETERS
blankFrames = 3
frameRate = 2e6
outputStep = 1/frameRate
specOutputFrames = 128
solveStep = 0.050e-6
outputFreq = int(outputStep/solveStep)
betaDamping = 1e-8
bVisLDamp = 0.0
bVisQDamp = 0.0

# SPECIMEN
specElemSize = 0.25e-3
specLength = 70e-3
specHeight = 45e-3
specThick = 3e-3

# CFRP - Onera UD0 plate 1
specAngle = 45
specRho = 1570
specE11 = 135e9
specE22 = 8.0e9
specNu12= 0.32
specG12 = 4.0e9

# LOADING PULSE DEFINITION
pulseStart = blankFrames*outputStep
pulseTriangle = 1
pulseHalfHeight = 0
pulseDuration = 18e-6
pulseRise = 9e-6	
pulseFall = 9e-6
pulsePressure = 300e6	# Pa

# Total Simulation Time Calculation
guideOutputFrames = 0
totalOutputSteps = int(specOutputFrames)
totalSimTime = totalOutputSteps*outputStep

#----------------------------------------------------------------------------
# Model and viewport definitions

mymodel=mdb.Model(name=jobFileName)

myviewport=session.Viewport(name='Pulse Model 2D',origin=(0.0, 0.0))
myviewport.makeCurrent()
myviewport.maximize()

#----------------------------------------------------------------------------
# Part Definitions

import part
myviewport.partDisplay.geometryOptions.setValues(
        referenceRepresentation=ON)

# SPECIMEN
s1 = mymodel.ConstrainedSketch(name='profile', 
        sheetSize=200.0)
g, v, d, c = s1.geometry, s1.vertices, s1.dimensions, s1.constraints
s1.setPrimaryObject(option=STANDALONE)
s1.rectangle(point1=(0.0, -specHeight/2), point2=(specLength, specHeight/2))

part_specimen = mymodel.Part(name='Specimen', dimensionality=TWO_D_PLANAR, 
        type=DEFORMABLE_BODY)
part_specimen.BaseShell(sketch=s1)
s1.unsetPrimaryObject()
myviewport.setValues(displayedObject=part_specimen)
del mymodel.sketches['profile']

#----------------------------------------------------------------------------
# Assembly Definition

import assembly
myassembly = mymodel.rootAssembly

# Create an instance of the specimen
specimen_inst = myassembly.Instance(name='SpecimenInstance', part=part_specimen, dependent=ON)

#----------------------------------------------------------------------------
# Material Definition

specMat = mymodel.Material(name='specMat')
specMat.Density(table=((specRho, ), ))
specMat.Elastic(type=LAMINA, table=((specE11, specE22, specNu12, specG12, 0.0, 0.0,), ))
specMat.Damping(beta=betaDamping)

part_specimen.DatumCsysByThreePoints(name='Datum csys-1', coordSysType=CARTESIAN, origin=(
	0.0, 0.0, 0.0), line1=(1.0, 0.0, 0.0), line2=(0.0, 1.0, 0.0))

#----------------------------------------------------------------------------
# Section

# Create the section		
myassembly.regenerate()    
section_specimen = mymodel.HomogeneousSolidSection(name='Section-Specimen', 
	material='specMat', thickness=specThick)
	
# Define regions
region_specimen = (part_specimen.faces,)

# Co-ords for orthotropic material
orientation = part_specimen.datums[2]

part_specimen.MaterialOrientation(region=region_specimen, 
	orientationType=SYSTEM, axis=AXIS_3, localCsys=orientation, fieldName='', 
	additionalRotationType=ROTATION_ANGLE, angle=specAngle, 
	additionalRotationField='', stackDirection=STACK_3) 

# Section Assignment	
part_specimen.SectionAssignment(region=region_specimen, sectionName='Section-Specimen', 
	offset=0.0, offsetType=MIDDLE_SURFACE, offsetField='', thicknessAssignment=FROM_SECTION)

#----------------------------------------------------------------------------
# Loading pulse shape definition

import load
if pulseTriangle:
	t1 = pulseStart
	t2 = t1+pulseRise
	t3 = t1+pulseDuration
	mymodel.TabularAmplitude(name='Amp-1', timeSpan=STEP,smooth=SOLVER_DEFAULT, 
		data=((0.0, 0.0), (t1, 0.0), (t2, 1.0), (t3, 0.0)))
else:
	t1 = pulseStart
	t2 = t1+pulseRise
	t3 = t1+pulseDuration-pulseFall
	t4 = t1+pulseDuration
	mymodel.TabularAmplitude(name='Amp-1', timeSpan=STEP,smooth=SOLVER_DEFAULT, 
		data=((0.0, 0.0), (t1, 0.0), (t2, 1.0), (t3, 1.0), (t4, 0.0)))


#----------------------------------------------------------------------------
# Mesh

# Seed parts with element edge size
part_specimen.seedPart(size=specElemSize, deviationFactor=0.1, minSizeFactor=0.1)
	
# Element type definition
elementType=mesh.ElemType(elemCode=CPS4R, elemLibrary=EXPLICIT,secondOrderAccuracy=OFF, 
hourglassControl=DEFAULT,  distortionControl=DEFAULT)

# Assign element type to parts
part_specimen.setElementType(regions=(region_specimen,),elemTypes=(elementType,))

# Generate the mesh
part_specimen.generateMesh()
myviewport.partDisplay.setValues(mesh=ON)
myviewport.assemblyDisplay.setValues(mesh=ON)
myviewport.assemblyDisplay.meshOptions.setValues(meshTechnique=ON)

# Define element and node sets for output
specimen_el = part_specimen.elements
specimen_nd = part_specimen.nodes

tol = specElemSize/2
specimen_elements = specimen_el.getByBoundingBox(0,-specHeight/2-tol,-tol,specLength+tol,specHeight/2+tol,tol)
part_specimen.Set(name='Set_elements_specimen', elements=specimen_elements)

specimen_nodes = specimen_nd.getByBoundingBox(0,-specHeight/2-tol,-tol,specLength+tol,specHeight/2+tol,tol)
part_specimen.Set(name='Set_node_specimen', nodes=specimen_nodes)

# Show the mesh
myviewport.assemblyDisplay.setValues(mesh=ON)
myviewport.assemblyDisplay.meshOptions.setValues(meshTechnique=ON)

#----------------------------------------------------------------------------
# Explicit Dynamics - Load Step Definition

name_step='Impact'
mymodel.ExplicitDynamicsStep(name=name_step, previous='Initial', 
	timePeriod=totalSimTime, description='Inertial Impact',
	timeIncrementationMethod=FIXED_USER_DEFINED_INC, userDefinedInc=solveStep)
mymodel.steps['Impact'].setValues(
    timeIncrementationMethod=AUTOMATIC_GLOBAL, scaleFactor=1.0, 
    maxIncrement=solveStep)
step_impact=mymodel.steps[name_step]
step_impact.setValues(nlgeom=OFF)

# Specify bulk viscosity
step_impact.setValues(linearBulkViscosity=bVisLDamp, 
	quadBulkViscosity=bVisQDamp)

myviewport.assemblyDisplay.setValues(loads=ON, bcs=ON, 
	predefinedFields=ON, connectors=ON, adaptiveMeshConstraints=OFF)

#----------------------------------------------------------------------------
# Explicit Dynamics - Pressure Pulse

# Edges of each part
edge_specimen = specimen_inst.edges 

# Specimen Contact Face
if pulseHalfHeight:
	X1 = specLength-tol
	Y1 = -specHeight/2
	Z1 = -tol
	X2 = specLength+tol
	Y2 = 0+tol
	Z2 = tol
else:
	X1 = specLength-tol
	Y1 = -specHeight/2
	Z1 = -tol
	X2 = specLength+tol
	Y2 = specHeight/2
	Z2 = tol

ed_specimen=edge_specimen.getByBoundingBox(X1,Y1,Z1,X2,Y2,Z2)

# Create the surfaces
surf_specimen=myassembly.Surface(name='Surf-Specimen',side1Edges=ed_specimen)

# Define the pressure pulse on the specimen edge
mymodel.Pressure(name='Load-1', createStepName='Impact', 
    region=surf_specimen, distributionType=UNIFORM, field='', magnitude=pulsePressure, 
    amplitude='Amp-1')

myviewport.assemblyDisplay.setValues(interactions=OFF, 
	constraints=OFF, connectors=OFF, engineeringFeatures=OFF)

#----------------------------------------------------------------------------
# Explicit Dynamics - Job and Output Definition

# Field Outputs
if extractDispEnerOnly:
	mymodel.fieldOutputRequests['F-Output-1'].setValues(variables=('UT','S','COORD'))
else:
	mymodel.fieldOutputRequests['F-Output-1'].setValues(variables=(
		'S', 'E', 'UT', 'AT','COORD'))
mymodel.fieldOutputRequests['F-Output-1'].setValues(timeInterval=outputStep,timeMarks=ON)

# History Outputs - Energy
mymodel.HistoryOutputRequest(name='H-Output-1', 
    createStepName='Impact', variables=('ALLAE', 'ALLDC', 'ALLDMD', 
    'ALLIE', 'ALLKE', 'ALLSE', 'ALLVD', 'ALLWK', 'ALLCW', 'ALLPW', 'ETOTAL'))
mymodel.historyOutputRequests['H-Output-1'].setValues(timeInterval=outputStep)

# Define the Job
Impact=mdb.Job(name=jobFileName, model=jobFileName, description='', type=ANALYSIS, 
	atTime=None, waitMinutes=0, waitHours=0, queue=None, memory=90, 
	memoryUnits=PERCENTAGE, explicitPrecision=SINGLE, 
	nodalOutputPrecision=SINGLE, echoPrint=OFF, modelPrint=OFF, 
	contactPrint=OFF, historyPrint=OFF, userSubroutine='', scratch='', 
	resultsFormat=ODB, parallelizationMethodExplicit=DOMAIN, numDomains=1, 
	activateLoadBalancing=False, multiprocessingMode=DEFAULT, numCpus=1)
print 'Job created. \n'

if not os.path.exists(directory+extension+jobFileName):
	os.makedirs(directory+extension+jobFileName)
# Set as working directory
os.chdir(directory+extension+jobFileName)

# Submit the job
Impact.submit(consistencyChecking=OFF)
print'Job '+jobFileName+' submitted'+'\n'+''

# Wait for job completion
print'Waiting for '+jobFileName+' to finish'+'\n'+'   '
Impact.waitForCompletion()

print jobFileName+' FINISHED'+'\n'+'   '

#----------------------------------------------------------------------------